"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, List
from lancedb.pydantic import PYDANTIC_VERSION
from .base import EmbeddingFunction
from .registry import register
from .utils import TEXT

@register("huggingface")
class TransformersEmbeddingFunction(EmbeddingFunction):
    """
    An embedding function that can use any model from the transformers library.

    Parameters:
    ----------
    name : str
        The name of the model to use. This should be a model name that can be loaded
        by transformers.AutoModel.from_pretrained. For example, "bert-base-uncased".
        default: "colbert-ir/colbertv2.0""
    device : str
        The device to use for the model. Default is "cpu".
    show_progress_bar : bool
        Whether to show a progress bar when loading the model. Default is True.
    trust_remote_code : bool
        Whether or not to allow for custom models defined on the HuggingFace
        Hub in their own modeling files. This option should only be set to True
        for repositories you trust and in which you have read the code, as it
        will execute code present on the Hub on your local machine.

    to download package, run :
        `pip install transformers`
    you may need to install pytorch as well - `https://pytorch.org/get-started/locally/`

    """
    name: str = ...
    device: str = ...
    trust_remote_code: bool = ...
    _tokenizer: Any = ...
    _model: Any = ...
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    if PYDANTIC_VERSION.major < 2:
        class Config:
            keep_untouched = ...
        
        
    else:
        model_config = ...
    def ndims(self): # -> int:
        ...
    
    def compute_query_embeddings(self, query: str, *args, **kwargs) -> List[np.array]:
        ...
    
    def compute_source_embeddings(self, texts: TEXT, *args, **kwargs) -> List[np.array]:
        ...
    


@register("colbert")
class ColbertEmbeddings(TransformersEmbeddingFunction):
    """
    An embedding function that uses the colbert model from the huggingface library.

    Parameters:
    ----------
    name : str
        The name of the model to use. This should be a model name that can be loaded
        by transformers.AutoModel.from_pretrained. For example, "bert-base-uncased".
        default: "colbert-ir/colbertv2.0""

    to download package, run :
        `pip install transformers`
    you may need to install pytorch as well - `https://pytorch.org/get-started/locally/`

    """
    name: str = ...
    def __init__(self, *args, **kwargs) -> None:
        ...
    


